<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE xsl:stylesheet [
<!ENTITY nbsp '&#160;' >
]>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method="html" indent="no"/>
<!-- Input Parameters -->
<xsl:param name="PrintUsage" select="''"/>
<xsl:param name="WebRootPath" select="''"/>

<!-- Print Usage-->
<xsl:template name="Usage">
#
# When running the XSLT, you can provide the following parameters:
#  - PrintUsage ${ANY_STRING}: Print this error message
#
</xsl:template>

<!-- Process columns -->
<xsl:template name="InsertCell">
  <xsl:param name="class"/>
  <xsl:param name="value" select="."/>
  <xsl:element name="td">
   <xsl:if test="string($class)">
    <xsl:attribute name="class">
     <xsl:value-of select="$class"/>
    </xsl:attribute>
   </xsl:if>
   <xsl:choose>
    <xsl:when test="string($value)">
      <xsl:choose>
       <xsl:when test="string($class) = 'float'">
        <xsl:value-of select="number($value)"/>
       </xsl:when>
       <xsl:when test="string($class) = 'percentage'">
        <xsl:value-of select="number($value)"/>
       </xsl:when>
       <xsl:otherwise>
        <xsl:value-of select="$value"/>
       </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:otherwise>&nbsp;</xsl:otherwise>
   </xsl:choose>
  </xsl:element>
</xsl:template> 

<!-- Translate Error Reason -->
<xsl:template name="TranslateReason">
 <xsl:variable name="reason" select="string(.)"/>
 <xsl:choose>
  <xsl:when test="$reason = 'SOURCE_SRM_ERROR'">Source&nbsp;SRM</xsl:when>
  <xsl:when test="$reason = 'DEST_SRM_ERROR'">Dest&nbsp;SRM</xsl:when>
  <xsl:when test="$reason = 'AGENT_ERROR'">Other</xsl:when>
  <xsl:when test="$reason = 'TRANSFER_ERROR'">Transfer</xsl:when>
  <xsl:when test="$reason = 'SECURITY_ERROR'">Security</xsl:when>
  <xsl:when test="$reason = 'NETWORK_ERROR'">Network</xsl:when>
  <xsl:when test="$reason = 'CONFIGURATION_ERROR'">Configuration</xsl:when>
  <xsl:when test="$reason = 'CONSISTENCY_ERROR'">Consistency</xsl:when>
  <!-- FTS 2.0 new error codes -->
  <xsl:when test="$reason = 'SOURCE_PREP_SRM_ERROR'">Source&nbsp;SRM:&nbsp;Prep</xsl:when>
  <xsl:when test="$reason = 'SOURCE_FINAL_SRM_ERROR'">Source&nbsp;SRM:&nbsp;Final</xsl:when>
  <xsl:when test="$reason = 'DEST_PREP_SRM_ERROR'">Dest&nbsp;SRM:&nbsp;Prep</xsl:when>
  <xsl:when test="$reason = 'DEST_FINAL_SRM_ERROR'">Dest&nbsp;SRM:&nbsp;Final</xsl:when>
  <xsl:when test="$reason = 'REMOTE_ERROR'">Connection</xsl:when>
  <xsl:when test="$reason = 'CATALOG_ERROR'">Catalog</xsl:when>
  <xsl:when test="$reason = 'SOURCE_CATALOG_ERROR'">Source&nbsp;Catalog</xsl:when>
  <xsl:when test="$reason = 'SOURCE_CATALOG_RESOLUTION_ERROR'">Source&nbsp;Catalog:&nbsp;Res</xsl:when>
  <xsl:when test="$reason = 'DEST_CATALOG_ERROR'">Dest&nbsp;Catalog</xsl:when>
  <xsl:when test="$reason = 'DEST_CATALOG_RESOLUTION_ERROR'">Dest&nbsp;Catalog:&nbsp;Res</xsl:when>
  <xsl:when test="$reason = 'DEST_CATALOG_REGISTRATION_ERROR'">Dest&nbsp;Catalog:&nbsp;Reg</xsl:when>
  <xsl:otherwise>
   <xsl:value-of select="$reason"/>
  </xsl:otherwise>
 </xsl:choose>
</xsl:template>

<!-- Process column tag -->
<xsl:template match="vo" mode="data" priority="1">
  <xsl:call-template name="InsertCell">
    <xsl:with-param name="class">text</xsl:with-param>
    <xsl:with-param name="value"><xsl:value-of select="@name"/></xsl:with-param>
  </xsl:call-template>
    
  <xsl:choose>
   <xsl:when test="/report/details/type = 'transfer'"> 
    <!-- Get the total number of attempts -->
    <xsl:variable name="total" select="number(./num_success) + number(./num_failure)"/>
    <xsl:call-template name="InsertCell">
      <xsl:with-param name="class">integer</xsl:with-param>
      <xsl:with-param name="value" select="$total"/>
    </xsl:call-template>
    <!-- Get the failure rate -->
    <xsl:call-template name="InsertCell">
      <xsl:with-param name="class">percentage</xsl:with-param>
      <xsl:with-param name="value" select="./failure_rate"/>
    </xsl:call-template>
    <!-- Get the failure ratio -->
<!--    <xsl:variable name="succ_vs_fail">
      <xsl:value-of select="num_success"/>/<xsl:value-of select="num_failure"/>
    </xsl:variable>
    <xsl:call-template name="InsertCell">
      <xsl:with-param name="class">text</xsl:with-param>
      <xsl:with-param name="value" select="$succ_vs_fail"/>
    </xsl:call-template>
-->
   </xsl:when>
   <xsl:when test="/report/details/type = 'file'"> 
    <!-- Get the total number of attempts -->
    <xsl:variable name="total" select="number(./first_attempt) + number(./second_attempt) + number(./third_attempt) + number(./failed_files)"/>
    <xsl:call-template name="InsertCell">
      <xsl:with-param name="class">integer</xsl:with-param>
      <xsl:with-param name="value" select="$total"/>
    </xsl:call-template>
    <!-- Get the failure rate -->
    <xsl:call-template name="InsertCell">
      <xsl:with-param name="class">percentage</xsl:with-param>
      <xsl:with-param name="value" select="round(100 * number(./failed_files) div (number(./first_attempt) + number(./second_attempt) + number(./third_attempt) + number(./failed_files)))"/>
    </xsl:call-template>
    <!-- Get the FTS efficiency -->
    <xsl:call-template name="InsertCell">
      <xsl:with-param name="class">percentage</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
         <xsl:when test="(number(./second_attempt) + number(./third_attempt) + number(./further_attempts)) = 0">
           <xsl:value-of select="number(0)"/>
         </xsl:when>
         <xsl:otherwise>
           <xsl:value-of select="round(100 * (number(./second_attempt) + number(./third_attempt) + number(./further_attempts)) div ((number(./second_attempt) + number(./third_attempt)) + number(./further_attempts) + number(./failed_files)))"/>
         </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
   </xsl:when>
   <xsl:otherwise>
    <!-- other report type specific fields -->
   </xsl:otherwise>
  </xsl:choose>

  <!-- For all the others, apply the template  -->
  <xsl:for-each select=".">
   <!-- Apply template to inner values-->
   <xsl:apply-templates mode="data"/>
  </xsl:for-each>
</xsl:template> 
<xsl:template match="failure_rate" mode="data" priority="1">
<!--Ignore it since it has been reordered -->
</xsl:template> 
<xsl:template match="score" mode="data" priority="1">
<!--Ignore it: score is used only for ordering and coloring -->
</xsl:template> 
<xsl:template match="num_success | num_failure" mode="data" priority="1">
  <xsl:call-template name="InsertCell">
    <xsl:with-param name="class">integer</xsl:with-param>
  </xsl:call-template>
</xsl:template> 
<xsl:template match="first_failure_rate | second_failure_rate" mode="data" priority="1">
  <xsl:call-template name="InsertCell">
    <xsl:with-param name="class">percentage</xsl:with-param>
  </xsl:call-template>
</xsl:template> 
<xsl:template match="avg_filesize | duration | tx_rate | sum_effective_bytes | sum_bytes" mode="data" priority="1">
  <xsl:call-template name="InsertCell">
    <xsl:with-param name="class">float</xsl:with-param>
  </xsl:call-template>
</xsl:template> 
<xsl:template match="first_failure_cause | second_failure_cause" mode="data" priority="1">
  <xsl:call-template name="InsertCell">
    <xsl:with-param name="class">text</xsl:with-param>
    <xsl:with-param name="value"><xsl:call-template name="TranslateReason"/></xsl:with-param>
  </xsl:call-template>
</xsl:template> 
<xsl:template match="vo/*" mode="data" priority="0">
  <xsl:call-template name="InsertCell"/>
</xsl:template>

<!-- Define Channel Criticality -->
<xsl:template name="SetChannelStatus">
   <xsl:param name="scope"/>
   <xsl:param name="node" select="."/>
   <xsl:variable name="score">
    <xsl:choose>
     <xsl:when test="string($node/score)"><xsl:value-of select="number($node/score)"/></xsl:when>
     <xsl:otherwise><xsl:value-of select="number(0)"/></xsl:otherwise>
    </xsl:choose>
   </xsl:variable>
   <xsl:variable name="prefix">
    <xsl:choose>
     <xsl:when test="string($scope)"><xsl:value-of select="$scope"/>-</xsl:when>
     <xsl:otherwise/>
    </xsl:choose>
   </xsl:variable>
   <xsl:choose>
    <xsl:when test="$score &lt;= 0.25">
     <xsl:attribute name="class"><xsl:value-of select="$prefix"/>critical</xsl:attribute>
    </xsl:when>
    <xsl:when test="$score &lt;= 0.50">
     <xsl:attribute name="class"><xsl:value-of select="$prefix"/>error</xsl:attribute>
    </xsl:when>
    <xsl:when test="$score &lt;= 0.80">
     <xsl:attribute name="class"><xsl:value-of select="$prefix"/>warn</xsl:attribute>
    </xsl:when>
    <xsl:when test="$score &lt;= 0.90">
     <xsl:attribute name="class"><xsl:value-of select="$prefix"/>normal</xsl:attribute>
    </xsl:when>
    <xsl:when test="$score &lt;= 1">
     <xsl:attribute name="class"><xsl:value-of select="$prefix"/>good</xsl:attribute>
    </xsl:when>
    <xsl:otherwise>
     <!-- No class to apply-->
    </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<!-- Process channel tag -->
<xsl:template match="channel">
  <xsl:for-each select="vo">
   <xsl:sort select="@name" order="ascending" data-type="text"/>
   <xsl:element name="tr">
    <!-- set the row cell-->
    <xsl:choose>
     <xsl:when test="@name = '[All]'">
      <!-- Set the channel status-->
      <xsl:call-template name="SetChannelStatus"/>
       <td class="text">
        <xsl:element name="a">
         <xsl:attribute name="onClick">toggle_display('<xsl:value-of select="../@name"/>')</xsl:attribute>
         <xsl:value-of select="../@name"/>
        </xsl:element>
       </td>
     </xsl:when>
     <xsl:otherwise>
      <!-- Set the channel status-->
      <xsl:call-template name="SetChannelStatus">
       <xsl:with-param name="scope">vo</xsl:with-param>
      </xsl:call-template>
      <xsl:attribute name="name"><xsl:value-of select="../@name"/></xsl:attribute>
      <xsl:element name="td">
       <xsl:call-template name="SetChannelStatus">
        <xsl:with-param name="node" select="../vo[@name = '[All]']"/>
       </xsl:call-template>
       &nbsp;
      </xsl:element>
     </xsl:otherwise>
    </xsl:choose>
    <xsl:apply-templates mode="data" select="."/>
   </xsl:element>
  </xsl:for-each>
</xsl:template> 

<!-- Column Header -->
<xsl:template match="vo" mode="header" priority="1"><th>VO Name</th></xsl:template> 
<!-- transfer reports -->
<xsl:template match="num_success" mode="header" priority="1"><th># Succ.</th></xsl:template> 
<xsl:template match="num_failure" mode="header" priority="1"><th># Fail.</th></xsl:template> 
<xsl:template match="failure_rate" mode="header" priority="1"></xsl:template>
<xsl:template match="first_failure_cause" mode="header" priority="1"><th>1st Failure Reason</th></xsl:template>
<xsl:template match="first_failure_rate" mode="header" priority="1"><th>% 1st Failure Reason</th></xsl:template>
<xsl:template match="second_failure_cause" mode="header" priority="1"><th>2nd Failure Reason</th></xsl:template>
<xsl:template match="second_failure_rate" mode="header" priority="1"><th>% 2nd Failure Reason</th></xsl:template>
<xsl:template match="avg_filesize" mode="header" priority="1"><th>Avg. Size (GiB)</th></xsl:template>
<xsl:template match="duration" mode="header" priority="1"><th>Avg. Duration (sec)</th></xsl:template>
<xsl:template match="tx_rate"  mode="header" priority="1"><th>Avg. Tx Rate (MB/sec)</th></xsl:template>
<xsl:template match="sum_effective_bytes" mode="header" priority="1"><th>Eff. Tx Bytes (GiB)</th></xsl:template>
<xsl:template match="sum_bytes" mode="header" priority="1"><th>Tx Bytes (GiB)</th></xsl:template>
<!-- File reports -->
<xsl:template match="first_attempt" mode="header" priority="1"><th>1st Attempt</th></xsl:template> 
<xsl:template match="second_attempt" mode="header" priority="1"><th>2nd Attempt</th></xsl:template> 
<xsl:template match="third_attempt" mode="header" priority="1"><th>3rd Attempt</th></xsl:template> 
<xsl:template match="further_attempts" mode="header" priority="1"><th>&gt;3 Attempts</th></xsl:template> 
<xsl:template match="failed_files" mode="header" priority="1"><th>Failed</th></xsl:template> 
<xsl:template match="score" mode="header" priority="0"></xsl:template> 
<xsl:template match="*" mode="header" priority="0"><th>?</th></xsl:template> 

<!-- Process data tag -->
<xsl:template match="data">
  <div>
    <table id="report_table" class="report-table">
      <xsl:if test="./channel">
       <!--Table header -->
       <tr>
        <th>Channel Name</th> 
        <xsl:apply-templates mode="header" select="./channel[1]/vo[1]"/>
        <xsl:choose>
         <xsl:when test="/report/details/type = 'transfer'"> 
        <th>Total</th>
        <th>% Failures</th>
         </xsl:when>
         <xsl:when test="/report/details/type = 'file'"> 
        <th>Total</th>
        <th>% Failures</th>
        <th>Retry Eff.</th>
         </xsl:when>
         <xsl:otherwise>
         <!-- Other report type specific columns -->
         </xsl:otherwise>
        </xsl:choose>
        <xsl:for-each select="./channel[1]/vo[1]/*">
         <xsl:apply-templates mode="header" select="."/>
        </xsl:for-each>       
       </tr>
      </xsl:if>
      <!-- process channel statistics, reordering based on the channel overall failure rate-->
      <xsl:for-each select="channel">
        <xsl:sort select="vo[@name = '[All]']/score" order="ascending" data-type="number"/>
        <xsl:apply-templates select="."/>
      </xsl:for-each>
    </table>
    <p>Click on the Channel Name to show the VO details</p>
  </div>
</xsl:template> 

<!-- Process details tag -->
<xsl:template match="details">
  <div class="report-details">
    <p><xsl:value-of select="string(description)"/> managed by <b><xsl:value-of select="string(fts/@instance)"/></b></p>
    <p>Between <b><xsl:value-of select="interval/@between"/></b> and <b><xsl:value-of select="interval/@and"/></b></p>
  </div>
  <div class="report-button">
    <div style="float:right">
      <a id="show_all" onClick="show_all()">Show VO details</a>
    </div>
    <div id="filter">
     <form name="filter" method="GET" action="">
      <input type="text" name="pattern" value="" />
      <input type="button" value="Filter" onClick="filter_channels(this.form.pattern.value)"/>
     </form>
    </div>
  </div>
</xsl:template> 

<!-- Put the Disclaimer -->
<xsl:template name="InsertDisclaimer">
  <div id="disclaimer">
   <h3>Disclaimer</h3>
    <p>This page contains a report generated from information stored in the FTS Database and is intended for reporting purposes only. Since the format will probably change in the future, it's therefore recommended not to use parsing robots on it.</p>
  </div>
</xsl:template> 

<!-- Put the Legend -->
<xsl:template name="InsertLegend">
  <div id="legend">
   <h3>Legend</h3>
  <xsl:choose>
   <xsl:when test="/report/details/type = 'transfer'"> 
   <table>
    <tr>
     <th>Column Name</th><th>Description</th>
    </tr>
    <tr>
     <td>Total</td><td>The total number of transfer attempts in the given period</td>
    </tr>
    <tr>
     <td>% Failures</td><td>The percentage of failures</td>
    </tr>
    <tr>
     <td># Succ.</td><td>The number of successful transfers</td>
    </tr>
    <tr>
     <td># Fail.</td><td>The number of failed transfer attemps</td>
    </tr>
    <tr>
     <td>1st Failure Reason</td><td>The most frequent failure reason observed</td>
    </tr>
    <tr>
     <td>% 1st Failure Reason</td><td>The ratio of the most frequent observed failure over all the failures</td>
    </tr>
    <tr>
     <td>2nd Failure Reason</td><td>The second most frequent failure reason observed</td>
    </tr>
    <tr>
     <td>% 2nd Failure Reason</td><td>The ratio of the second most frequent observed failure over all the failures</td>
    </tr>
    <tr>
     <td>Avg. Size</td><td>The average size of the successfully transferred files, in GigaByte (2^30)</td>
    </tr>
    <tr>
     <td>Avg. Duration</td><td>The average duration of the successfully transfers, in seconds, observed by the FTS. This value includes also the SRM interactions</td>
    </tr>
    <tr>
     <td>Avg. Tx Rate</td><td>The observed average transfer rate of the successful transfers, in MegaByte/seconds. This value is computed as (File Size)/(Duration), therefore it takes into account the SRM interactions</td>
    </tr>
    <tr>
     <td>Eff. Tx Bytes</td><td>The sum of the effective bytes transferred in the given period, in GigaBytes (2^30). This value doesn't take into account the bytes transferred during failed attempts</td>
    </tr>
    <tr>
     <td>Tx Bytes</td><td>The sum of the bytes transferred in the given period, in GigaBytes (2^30). This value consider also the bytes transferred during failed attempts</td>
     </tr>
   </table>
   <table>
    <tr><th>Failure Reason</th><th>Cause</th>
    </tr>
    <tr>
     <td>Source SRM</td><td>Error in interacting with the Source SRM. This error category includes: 
      <ul>
       <li>Source SRM not reachable</li>
       <li>HTTP time out in contacting the source SRM</li>
       <li>Failed Srm Get operation</li>
       <li>Source File not found</li>
       <li>Srm Get time out</li>
       <li>Failed Srm Get Done</li>
      </ul>
      Source SRM error category may also specify the error phase:
      <ul>
       <li>Prep: the error occurred during the preparation, when retrieving the TURL from the SURL (PrepareToGet)</li>
       <li>Final: the error occurrend during the finalization, when releasing the SURL (ReleaseFiles)</li>
      </ul>
     </td>
    </tr>
    <tr>
      <td>Dest SRM</td><td>Error in interacting with the Destination SRM. This error category includes:
       <ul>
        <li>Destination SRM not reachable</li>
        <li>HTTP time out in contacting the destination SRM</li>
        <li>Failed Srm Put operation</li>
        <li>Destination File already exists (for dCache or DPM)</li>
        <li>Srm Put time out</li>
        <li>Failed Srm Put Done</li>
       </ul>
      Dest SRM error category may also specify the error phase:
      <ul>
       <li>Prep: the error occurred during the preparation, when retrieving the TURL from the SURL (PreparteToPut)</li>
       <li>Final: the error occurrend during the finalization, when completing the Put request (PutDone)</li>
      </ul>
      </td>
    </tr>
    <tr>
      <td>Connection</td><td>Error in contacting one of the SRM. Please check if this error is common on other channels in orther to detect which is the site causing this problem 
      </td>
    </tr>
    <tr>
      <td>Transfer</td><td>Error in during the transfers:
       <ul>
        <li>All the errors of an SRMCopy channel (either in contacting the SRM or if the request fails)</li>
        <li>Mismatch between the source and destination file size after the transfer is completed</li>
        <li>GSIFTP failures</li>
        <li>Aborted GridFTP transfers</li>
       </ul> 
      </td>
    </tr>
    <tr>
      <td>Network</td><td>Network error in during the transfers:
       <ul>
        <li>Transfer markers timeout</li>
       </ul> 
      </td>
    </tr>
    <tr>
      <td>Security</td><td>Security errors, like:
       <ul>
        <li>Invalid User Credential stored in MyProxy</li>
        <li>Invalid User's passphrase specified</li>
        <li>Expired CRL</li>
        <li>Missing CA</li>
       </ul> 
      </td>
    </tr>
    <tr>
      <td>Other</td><td>Other error not classified in the previous categories</td>
    </tr>
   </table>
    </xsl:when>
   <xsl:when test="/report/details/type = 'file'"> 
   <table>
    <tr>
     <th>Column Name</th><th>Description</th>
    </tr>
    <tr>
     <td>Total</td><td>The total number of files handled in the given period</td>
    </tr>
    <tr>
     <td>% Failures</td><td>The percentage of file failures</td>
    </tr>
    <tr>
     <td>Retry Eff.</td><td>An indication of the FTS efficiency in retrying the files, computed as the number of files that were successfully transferred over the total number of files that failed at least once. This measure doesn't take into account files successfully completed at the first attempt. Please note that if a VO disabled the FTS retry, this number is 0</td>
    </tr>
    <tr>
     <td>1st Attempt</td><td>The number of files succeeded at the first attempt</td>
    </tr>
    <tr>
     <td>2nd Attempt</td><td>The number of files succeeded at the second attempt</td>
    </tr>
    <tr>
     <td>3rd Attempt</td><td>The number of files succeeded at the third attempt</td>
    </tr>
    <tr>
     <td>&gt;3 Attempts</td><td>The number of files succeeded after the third attempt</td>
    </tr>
    <tr>
     <td>Failed</td><td>The number of files failed</td>
    </tr>
   </table>
    </xsl:when>
    <xsl:otherwise>
    </xsl:otherwise>
  </xsl:choose>
  </div>
</xsl:template>
 
<!-- Start the matching -->
<xsl:template match="/report">
  <xsl:choose>
     <xsl:when test="string($PrintUsage) = 0">
       <xsl:call-template name="Usage"/>
     </xsl:when>
     <xsl:otherwise>
<html>
  <head>  
    <title>FTS Report</title>
    <xsl:element name="link">
     <xsl:attribute name="href"><xsl:value-of select="$WebRootPath"/>/report.css</xsl:attribute>
     <xsl:attribute name="type">text/css</xsl:attribute>
     <xsl:attribute name="rel" >stylesheet</xsl:attribute>
    </xsl:element>
    <xsl:element name="link">
     <xsl:attribute name="href"><xsl:value-of select="$WebRootPath"/>/images/fts.ico</xsl:attribute>
     <xsl:attribute name="type">image/x-icon</xsl:attribute>
     <xsl:attribute name="rel" >shortcut icon</xsl:attribute>
    </xsl:element>
    <xsl:element name="script">
     <xsl:attribute name="src" ><xsl:value-of select="$WebRootPath"/>/report.js</xsl:attribute>
     <xsl:attribute name="type">text/javascript</xsl:attribute>
    </xsl:element>
  </head>
  <body onLoad="fix_visibility()">
    <div id="main">
      <div style="float:right;">
       <xsl:element name="img">
         <xsl:attribute name="id">fts_logo</xsl:attribute>
         <xsl:attribute name="alt"></xsl:attribute>
         <xsl:attribute name="onLoad">fix_png_alpha(this);</xsl:attribute>
         <xsl:attribute name="src" ><xsl:value-of select="$WebRootPath"/>/images/shadow_text_big.png</xsl:attribute>
       </xsl:element>
      </div>
      <h1>FTS Report</h1>
      <xsl:call-template name="InsertDisclaimer"/>
      <div id="content">
        <xsl:apply-templates/>
      </div>
      <br/>
      <xsl:call-template name="InsertLegend"/>
    </div>
  </body>
</html>
     </xsl:otherwise>
   </xsl:choose>
</xsl:template>
</xsl:stylesheet>

